/***********************************************************************      
 *       
 * CONSOLE
 * initConsole : methode pour affichage du deroulement sur la console, parametre le debit (par defaut 9600);
 * 
 * CONTROLE
 * initControle : methode qui permet de definir deux leds (libre/occupe) et un bouton pour le reset, peux-etre utile pour un TCO     
 *          
 * COMMANDE
 * initComande : methode qui permet de definir une sortie pour des leds, une sortie en cas de passage libre 
 * et une sortie en cas de passage occupe, peut-etre utile pour commander des organes externes (feux de route, barrières).
 * Dans le cas d'un passage libre le programme desactive la sortie LED (etat LOW)
 * active la sortie libre (etat HIGH)
 * desactive la sortie occupee (etat LOW)
 * Dans le cas d'un passage occupe le programme active la sortie LED (etat HIGH)
 * desactive la sortie libre (etat LOW)
 * active la sortie occupee (etat HIGH)
 * 
 * zoneMultiPath passage (actionFreeZone,actionBusyZone) : Permet de creer ses propres routines de gestion d'un passage,
 * remplace la methode initCommand(). 
 * La premiere routine est pour le cas d'un passage libre et la seconde pour le cas d'un passage occupe.
 * 
 * 
 * La configuration:
 * elle consiste a fournir l'ensemble des broches utilisees, a determiner les zones du passage et les capteurs associes
 * et a regler le temps d'attente entre le passage occupe et le passage libre.
 * Il faut ensuite dans la methode setup pour chaque capteur appeler la methode :
 * initCapteur(brocheDuCapteur, numeroDeZone, nom);
 * les deux premiers parametres sont obligatoires, le nom est optionnel
 * 
 * l'execution consiste a appeler la methode Loop() dans la boucle du programme
 * 
 **********************************************************************************************************************/

#include <zoneMultiPath.h>                      // la bibliotheque pour le passage

zoneMultiPath passage;                          // a utiliser avec les methodes d'action par defaut du programme
//zoneMultiPath passage (actionFree,actionBusy);    // a utiliser si l'on definit ses propres methodes d'action

/************************************************************************
 * ***********************************************************************
 * CONFIGURATION
 * ************************************************************************
 ************************************************************************/

// pour le controle, si necessaire

#define LED_LIBRE 12                  // donner le numero de la broche de la led temoin zone libre   
#define LED_OCCUPE 13                 // donner le numero de la broche de la led temoin zone occupee
#define BOUTON_RESET A0               // donner le numero de la broche du bouton reset (mode pullup)

// pour les commandes, si necessaire
                           
#define COMMANDE_LED 11               // donner le numero de la broche de commande des leds (HIGH si occupe)            
#define COMMANDE_ZONE_LIBRE 9         // donner le numero de la broche de commande zone libre (HIGH si libre)       
#define COMMANDE_ZONE_OCCUPEE 10      // donner le numero de la broche de commande zone occupee (HIGH si occupe)


// declaration des zones parcourant le passage

#define NOMBRE_ZONES 2                  // donner le nombre total de zones parcourues

#define INDEX_ZONE1 0                     // index des zones
#define INDEX_ZONE2 1       

// declaration des capteurs

#define NOMBRE_CAPTEURS  6       // donner le nombre total de capteurs

#define VOIE1 "voie1"             // definir le nom de la voie (optionnel)
#define CAPTEUR_VOIE_1    2       // definir le numero de la broche de son capteur 
#define VOIE2 "voie2"
#define CAPTEUR_VOIE_2    3 
#define VOIE3 "voie3"          
#define CAPTEUR_VOIE_3    4
#define VOIE4 "voie4"           
#define CAPTEUR_VOIE_4    6 
#define VOIE5 "voie5"          
#define CAPTEUR_VOIE_5    7
#define VOIE6 "voie6"           
#define CAPTEUR_VOIE_6    8

// temps d'attente avant liberation apres une occupation, valeur par defaut 1000ms

#define ATTENTE 1000            // valeur a regler en fonction du type de capteur et de la longueur des trains


/**************************************************************************
 * 
 * La methode setup : initialise la configuration
 * 
***************************************************************************/

void setup() {
  passage.initConsole();                                                          // optionnel, si pas besoin a supprimer
  passage.initControl(LED_LIBRE,LED_OCCUPE,BOUTON_RESET);                         // optionnel, si pas besoin a supprimer
  passage.initCommand(COMMANDE_LED,COMMANDE_ZONE_LIBRE,COMMANDE_ZONE_OCCUPEE);    // optionnel, si pas besoin a supprimer
  passage.initRailroadCrossing(NOMBRE_CAPTEURS,NOMBRE_ZONES,ATTENTE);             // obligatoire, ATTENTE par defaut = 1000ms
  passage.initSensor(CAPTEUR_VOIE_1,LOW,INDEX_ZONE1,VOIE1);        // pour chaque voie donner son capteur, sa zone et son nom (facultatif)
  passage.initSensor(CAPTEUR_VOIE_2,LOW,INDEX_ZONE1,VOIE2);    
  passage.initSensor(CAPTEUR_VOIE_3,LOW,INDEX_ZONE2,VOIE3);   
  passage.initSensor(CAPTEUR_VOIE_4,LOW,INDEX_ZONE2,VOIE4);
  passage.initSensor(CAPTEUR_VOIE_5,LOW,INDEX_ZONE2,VOIE5);  
  passage.initSensor(CAPTEUR_VOIE_6,LOW,INDEX_ZONE2,VOIE6);
}

void loop() {
	passage.Loop();
}                                       // on boucle sur l'etat du passage

// methode optionnelle, permet de definir ses propres actions en cas de passage libre
// il faut utiliser l'instanciation zoneMultiPath passage (actionFree,actionBusy); 
void actionFree() {
	// mettre ici les actions a faire dans le cas d'un passage libre
	Serial.println(" on utilise la fonction actionFree du programme");
}

// methode optionnelle, permet de definir ses propres actions en cas de passage occupe
// il faut utiliser l'instanciation zoneMultiPath passage (actionFree,actionBusy); 
void actionBusy() {
	// mettre ici les actions a faire dans le cas d'un passage occupe
	Serial.println(" on utilise la fonction actionBusy du programme");
}


 
