#include "zoneMultiPath.h"

#include <Arduino.h>

zoneMultiPath::zoneMultiPath(){
_actionFree = NULL;
_actionBusy = NULL;
}

zoneMultiPath::zoneMultiPath(void (*actionFree)(), void (*actionBusy)()){

_actionFree = actionFree;
_actionBusy = actionBusy;
}


void zoneMultiPath::initConsole (int bauds){
            Serial.begin(bauds);
            console = true;
            }

void zoneMultiPath::initControl (int pinFreeled, int pinBusyLed, int pinButton) {
        control = true;
        freeLedPin = pinFreeled;
        busyLedPin = pinBusyLed;
        buttonPin = pinButton;
        pinMode(freeLedPin,OUTPUT); digitalWrite(freeLedPin,HIGH);
        pinMode(busyLedPin,OUTPUT); digitalWrite(busyLedPin,LOW);
        pinMode(buttonPin,INPUT_PULLUP);
}

void zoneMultiPath::initCommand (int pinLed, int pinFreeZone, int pinBusyZone) {
        command = true;
        lightPin = pinLed;
        freeZonePin = pinFreeZone;
        busyZonePin = pinBusyZone;
        pinMode(lightPin,OUTPUT); digitalWrite(lightPin,LOW);
        pinMode(freeZonePin,OUTPUT); digitalWrite(freeZonePin,HIGH);
        pinMode(busyZonePin,OUTPUT); digitalWrite(busyZonePin,LOW);
}

void zoneMultiPath::initRailroadCrossing (int sensorsNumber, int zoneNumber, int waiting){
        railroadCrossing = true;
        sensorsNB = sensorsNumber;
        zoneNB = zoneNumber;
        waitingTime = waiting;
        sensor = new Tsensor[sensorsNumber];
        busyZone = new Tzone [zoneNumber];

}

void zoneMultiPath::initSensor (int Pin, bool Active, int Zone, char* Name){
    if ((sensorIndex < sensorsNB)& (railroadCrossing)){
        sensor[sensorIndex].name = Name;
        sensor[sensorIndex].pin = Pin;
        sensor[sensorIndex].activeLowHigh = Active;
        pinMode(sensor[sensorIndex].pin,INPUT_PULLUP);
        sensor[sensorIndex].zoneNumber = Zone;
        sensor[sensorIndex].predState = !Active;
        busyZone[Zone].busy = false;
        busyZone[Zone].state = FREE;
        busyZone[Zone].entry_sensor_number = -1;
        busyZone[Zone].exit_sensor_number = -1;
       if (console) {
            Serial.print (" Sensor ");
            Serial.print(sensorIndex);
            Serial.print ("  ");
            Serial.print(sensor[sensorIndex].name);
            Serial.print(" zone");
            Serial.print(sensor[sensorIndex].zoneNumber);
            Serial.print(" : pin = ");
            Serial.print(sensor[sensorIndex].pin);
            Serial.println(" , state  : FREE");
           }
       sensorIndex++;
    }
}

void zoneMultiPath::Loop() {
        if (control) { testResetButton();}
          boolean busy = false;
          byte sensorStatus;
          for (int i =0; i<sensorsNB;i++) {
          sensorStatus = digitalRead(sensor[i].pin);
          if (sensorStatus != sensor[i].predState ){              // si le capteur a changer d'etat
          sensor[i].predState = sensorStatus;                 // on conserve le nouvel etat
          if (sensorStatus == sensor[i].activeLowHigh) {                          // si le capteur est actif
              if ((busyZone[sensor[i].zoneNumber].busy) &             // on passe de ENTREE -> SORTIE
                  (busyZone[sensor[i].zoneNumber].state == ENTRY)
                  & (busyZone[sensor[i].zoneNumber].entry_sensor_number != i)) { // capteur de sortie != capteur entree
                      if (console){
                       Serial.print(" exit to path "); Serial.println(sensor[i].name);
                      }
                       busyZone[sensor[i].zoneNumber].state = EXIT;      // on passe en zone SORTIE
                       busyZone[sensor[i].zoneNumber].exit_sensor_number = i;  // on garde le numero de capteur de sortie
                  }
              else {if (busyZone[sensor[i].zoneNumber].state == FREE) {   // on passe de LIBRE a ENTREE
                      if (console) {
                        Serial.print(" enter from path "); Serial.println(sensor[i].name);
                      }
                      busyZone[sensor[i].zoneNumber].state = ENTRY;      // on passe en zone ENTREE
                      busyZone[sensor[i].zoneNumber].busy = true;     // la zone est occupee
                      busyZone[sensor[i].zoneNumber].entry_sensor_number = i;  // on garde le numero du capteur d'entree
                      }
                    else {if (busyZone[sensor[i].zoneNumber].state == EXIT){  // on reste en zone SORTIE mais on  change de capteur de sortie
                             if(console) {
                              Serial.print(" exit to path "); Serial.println(sensor[i].name);
                             }
                             busyZone[sensor[i].zoneNumber].exit_sensor_number = i;  // on garde le numero de capteur de sortie
                           }
                        }
                   }
             }
     else { if ((busyZone[sensor[i].zoneNumber].state == EXIT)&
                (busyZone[sensor[i].zoneNumber].exit_sensor_number == i)) {
                    busyZone[sensor[i].zoneNumber].state = FREE;
                    busyZone[sensor[i].zoneNumber].busy = false;
                    busyZone[sensor[i].zoneNumber].entry_sensor_number = -1;
                    busyZone[sensor[i].zoneNumber].exit_sensor_number = -1;}
                  }
     }
     }
  if (checkBusyZone() == 0 ) {freeZoneManagement();}else {busyZoneManagement();}
}

void zoneMultiPath::freeZoneManagement() {
    if (freeCPT) {
              busyCPT = true;
               unsigned long waiting = millis() + waitingTime;
               while (millis() < waiting){
                  if (checkBusyZone()) {waiting = 0;break;}
                  }
              if (waiting != 0) {
                if (console) {
                   Serial.println(" railway Crossing free");
                }
                if (_actionFree == NULL) {actionFreeZone();}
                else {_actionFree();}
                }
        }
     freeCPT = false;
}

void zoneMultiPath::actionFreeZone() {
  if (command) {
        digitalWrite(lightPin, LOW);
        digitalWrite(freeZonePin, HIGH);
        digitalWrite(busyZonePin, LOW);
      }
  if (control) {
     digitalWrite(freeLedPin,HIGH);
     digitalWrite(busyLedPin,LOW);
    }
}

void zoneMultiPath::busyZoneManagement() {
       if (busyCPT) {
          freeCPT = true;
          if (console) {
            Serial.println(" railway Crossing busy");
            }
            if (_actionBusy == NULL) {actionBusyZone();}
            else {_actionBusy();}
        }
     busyCPT = false;
}

 void zoneMultiPath::actionBusyZone() {
   if (command) {
     digitalWrite(lightPin, HIGH);
     digitalWrite(busyZonePin, HIGH);
     digitalWrite(freeZonePin, LOW);
      }
    if (control) {
        digitalWrite(freeLedPin,LOW);
        digitalWrite(busyLedPin,HIGH);
    }
 }

byte zoneMultiPath::checkBusyZone () {
   byte result = 0;
   for (int i = 0; i<zoneNB;i++){result = result + busyZone[i].state;}
  return result;
}

void zoneMultiPath::testResetButton() {
     if(digitalRead(buttonPin) == LOW) {
     for (int i=0;i<sensorsNB;i++) {sensor[i].predState = LOW;}
     for (int i=0;i<zoneNB;i++) {
       busyZone[i].busy = false;
       busyZone[i].state = FREE;
       busyZone[i].entry_sensor_number = -1;
       busyZone[i].exit_sensor_number = -1;
     }
  }




}
